SystemOrganization addCategory: #'SqSync-Shared'!
SystemOrganization addCategory: #'SqSync-Client'!
SystemOrganization addCategory: #'SqSync-Server'!
SystemOrganization addCategory: #'SqSync-Tests'!

OrderedCollection subclass: #SySharedList
	instanceVariableNames: 'mutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Shared'!

!SySharedList commentStamp: 'mt 5/9/2010 17:04:27.753' prior: 0!
This is meant to be a thread-safe version of an OrderedCollection. See the "list" protocol to explore selectors which are thread-safe.!

!SySharedList methodsFor: 'list' stamp: 'mt 5/9/2010 16:52:24.122'!
add: anElement

	^ self mutex critical: [super add: anElement].! !

!SySharedList methodsFor: 'accessing - private' stamp: 'mt 5/9/2010 16:40:52.25'!
mutex

	^ mutex ifNil: [mutex := Mutex new]! !

!SySharedList methodsFor: 'list' stamp: 'mt 5/9/2010 16:53:19.735'!
remove: anElement ifAbsent: aBlock
	"#remove: should also be guarded now."
	
	^ self mutex critical: [super remove: anElement ifAbsent: aBlock].! !

Object subclass: #SyNetworkDataProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Shared'!

!SyNetworkDataProcessor commentStamp: 'mt 5/9/2010 17:01:16.718' prior: 0!
This is a place holder for the "P"" in our diagram.

Current contract: NO STATE - ONLY BEHAVIOR!

!SyNetworkDataProcessor class methodsFor: 'processing' stamp: 'mt 5/9/2010 17:02:23.913'!
processDataFrom: aStream
	"Hint: Expect stream API from SocketStream. Other Streams should work as well."! !

Object subclass: #SyTcpServer
	instanceVariableNames: 'listenSocket listenProcess connections connectionProcesses connectionChangesMonitor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Server'!

!SyTcpServer methodsFor: 'connection open/close' stamp: 'SR 5/10/2010 17:38:30.467'!
close

	"Closing listen part"
	listenProcess terminate.
	self listenSocket close.
	listenSocket := nil.
	
	"Closing connection part"
	self connectionProcesses do: [:proc | proc terminate].! !

!SyTcpServer methodsFor: 'waiting' stamp: 'SR 5/10/2010 18:25:52.291'!
connectionChangesMonitor

	^ connectionChangesMonitor! !

!SyTcpServer methodsFor: 'accessing - private' stamp: 'SR 5/10/2010 17:28:42.275'!
connectionProcesses

	^ connectionProcesses ifNil: [connectionProcesses := SySharedList new]! !

!SyTcpServer methodsFor: 'accessing - private' stamp: 'mt 5/9/2010 16:46:22.708'!
connections

	^ connections ifNil: [connections := SySharedList new]! !

!SyTcpServer methodsFor: 'waiting' stamp: 'SR 5/10/2010 18:26:19.094'!
initialize

	super initialize.
	
	connectionChangesMonitor := Monitor new.! !

!SyTcpServer methodsFor: 'testing' stamp: 'mt 5/9/2010 16:17:23.888'!
isListening

	^ self listenSocket isWaitingForConnection! !

!SyTcpServer methodsFor: 'connection open/close' stamp: 'SR 5/10/2010 18:09:12.044'!
listenOn: aPort

	self listenSocket listenOn: aPort backlogSize: 4.
	self startListenThread.! !

!SyTcpServer methodsFor: 'accessing - private' stamp: 'mt 5/9/2010 15:58:25.953'!
listenSocket

	^ listenSocket ifNil: [listenSocket := Socket newTCP]! !

!SyTcpServer methodsFor: 'accessing' stamp: 'mt 5/9/2010 17:12:59.484'!
port

	^ self listenSocket port! !

!SyTcpServer methodsFor: 'threads' stamp: 'SR 5/10/2010 18:26:38.848'!
startConnectionThreadFor: aTcpSocket

	self connectionProcesses add: (
		[
			[ aTcpSocket isConnected] whileTrue: [
				aTcpSocket waitForDataIfClosed: [
					self connections remove: aTcpSocket.
					self connectionChangesMonitor signalAll.
					^ self].
				SyNetworkDataProcessor processDataFrom: aTcpSocket.
			]
		] forkAt: 35)! !

!SyTcpServer methodsFor: 'threads' stamp: 'SR 5/10/2010 18:26:44.265'!
startListenThread
	
	listenProcess := [
		[self isListening] whileTrue: [
			(self listenSocket waitForAcceptFor: 10)
				ifNotNilDo: [:socket |
					| con |
					con := SyTcpSocket on: socket.
					self connections add: con.
					self startConnectionThreadFor: con.
					self connectionChangesMonitor signalAll.].
			]
		] forkAt: 35.! !

!SyTcpServer methodsFor: 'waiting' stamp: 'mt 5/10/2010 11:27:21.847'!
waitForNewConnectionFrom: nameOrAddressString port: aPort

	| remoteAddress |
	remoteAddress := NetNameResolver addressForName: nameOrAddressString timeout: 20.
	
	[
		self connections includes: [:tcpSocket | 
			tcpSocket remoteAddress = remoteAddress
			and: [tcpSocket remotePort = aPort]].
		Processor yield
	] whileFalse.! !

TestCase subclass: #SyCommunicationModuleTest
	instanceVariableNames: 'tcpServer tcpSocket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Tests'!

!SyCommunicationModuleTest class methodsFor: 'constants' stamp: 'mt 5/10/2010 11:06:45.568'!
defaultPort

	^ 22555! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'SR 5/10/2010 17:35:29.241'!
connectTCPClientToServer

	tcpSocket := SyTcpSocket
		connectToHost: '127.0.0.1'
		port: self class defaultPort.! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'SR 5/10/2010 17:36:17.01'!
disconnectTCPClientFromServer

	(tcpSocket notNil and: [tcpSocket isConnected])
		ifTrue: [tcpSocket close. tcpSocket := nil].! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'mt 5/10/2010 11:39:41.976'!
startTcpServer

	tcpServer ifNil: [tcpServer := SyTcpServer new].
	tcpServer isListening
		ifFalse: [tcpServer listenOn: self class defaultPort].! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'SR 5/10/2010 17:17:02.712'!
stopTcpServer

	(tcpServer notNil and: [tcpServer isListening])
		ifTrue: [tcpServer close].! !

!SyCommunicationModuleTest methodsFor: 'running' stamp: 'SR 5/10/2010 17:36:27.38'!
tearDown

	super tearDown.
	
	self
		disconnectTCPClientFromServer;
		stopTcpServer.! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'SR 5/10/2010 18:11:05.098'!
testTcpServerIncomingConnections

	| anotherClient |
	self
		startTcpServer;
		should: tcpServer connections isEmpty;
		connectTCPClientToServer;
		waitForServer;
		should: tcpServer connections size = 1.
	
	anotherClient := SyTcpSocket
		connectToHost: '127.0.0.1'
		port: self class defaultPort.
		
	self
		waitForServer;
		should: tcpServer connections size = 2;
		disconnectTCPClientFromServer;
		"waitForServer;"
		should: tcpServer connections size = 1.
	
	anotherClient close.
	
	self should: tcpServer connections isEmpty.! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'SR 5/10/2010 17:55:25.79'!
testTcpServerShutdown
	
	self
		startTcpServer;
		stopTcpServer;
		shouldnt: tcpServer isListening;
		should: tcpServer connections isEmpty;
		should: (Socket registry keys noneSatisfy: [:socket | socket isWaitingForConnection] ).! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'mt 5/10/2010 11:12:42.799'!
testTcpServerStartup

	| sut |
	sut := SyTcpServer new.
	
	self shouldnt: sut isListening.
	
	sut listenOn: self class defaultPort.
	
	self
		should: sut isListening;
		should: sut port = self class defaultPort;
		should: sut connections isEmpty.
		
	sut close.! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'SR 5/10/2010 17:59:17.408'!
testTcpSocketConnection

	| connectionFailed |
	connectionFailed := false.
	[ self connectTCPClientToServer. ]
	on: ConnectionTimedOut
	do: [ connectionFailed := true ].
	
	self
		should: connectionFailed;
		startTcpServer;
		connectTCPClientToServer;
		should: tcpSocket isConnected;
		waitForServer;
		should: (tcpServer connectionProcesses size = 1).! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'SR 5/10/2010 18:36:30.881'!
waitForServer

	"(Delay forSeconds: 1) wait."
	tcpServer connectionChangesMonitor wait.! !

SocketStream subclass: #SyTcpSocket
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Shared'!

!SyTcpSocket class methodsFor: 'instance creation' stamp: 'mt 5/10/2010 11:05:32.461'!
connectToHost: nameOrAddressString port: anInteger
	
	^ self
		openConnectionToHostNamed: nameOrAddressString
		port: anInteger! !

!SyTcpSocket methodsFor: 'accessing' stamp: 'mt 5/10/2010 11:23:48.023'!
remoteAddress

	^ self socket remoteAddress! !

!SyTcpSocket methodsFor: 'accessing' stamp: 'mt 5/10/2010 11:23:53.871'!
remotePort

	^ self socket remotePort! !

!SyTcpSocket methodsFor: 'waiting' stamp: 'mt 5/9/2010 17:23:15.848'!
waitForDataIfClosed: aBlock

	^ self socket waitForDataIfClosed: aBlock! !
